using System;
using System.Collections;
using System.Net.Sockets;
using System.Net;
using System.Text;
using System.Data;

namespace Slush.Net
{
	/// <summary>
	/// 
	/// </summary>
	public class HTTPSession
	{
		protected Hashtable cookies = new Hashtable(); 
		protected string hostName;
		protected IPAddress hostAddress;
		protected IPEndPoint hostEndpoint;

		public HTTPSession(string server)
		{
			hostName = server;
			hostAddress = Dns.Resolve(server).AddressList[0];
			hostEndpoint = new IPEndPoint(hostAddress, 80);
		}

		public void saveCookies(string file)
		{
			System.IO.StreamWriter str = System.IO.File.CreateText(file);
			foreach( DictionaryEntry entry in cookies )
			{
                str.WriteLine(entry.Key + "=" + entry.Value);				
			}
			str.Close();
		}

		public void clearCookies()
		{
			cookies.Clear();
		}


		public string getFile(string URL)
		{
			string header = "GET " + URL + " HTTP/1.1\r\n" +
					 "Host: " + hostName + "\r\n" +
					 getCookieString() + 
					 "Connection: Close\r\n\r\n";
			return getStringInternal(header, URL);
		}

		public string getFile(string refer, string URL)
		{
			string header = "GET " + URL + " HTTP/1.1\r\n" +
				"Host: " + hostName + "\r\n" +
				"Connection: Close\r\n" +
				getCookieString() + 
				"Referer: " + refer + "\r\n\r\n";
			return getStringInternal(header, URL);
		}

		public Byte[] getBinary(string URL)
		{
			string header = "GET " + URL + " HTTP/1.1\r\n" +
				"Host: " + hostName + "\r\n" +
				getCookieString() + 
				"Connection: Close\r\n\r\n";
			return getBinaryInternal(header, URL);
		}

		public Byte[] getBinary(string refer, string URL)
		{
			string header = "GET " + URL + " HTTP/1.1\r\n" +
				"Host: " + hostName + "\r\n" +
				"Connection: Close\r\n" +
				getCookieString() + 
				"Referer: " + refer + "\r\n\r\n";
			return getBinaryInternal(header, URL);
		}

		public string getFile(Hashtable post, string URL)
		{
			string postData = makePostString(post);
			string header = "POST " + URL + " HTTP/1.1\r\n" +
				"Host: " + hostName + "\r\n" +
				"Accept: */*\r\n" +
				"Connection: Close\r\n" +
				getCookieString() + 
				"Content-Type: application/x-www-form-urlencoded\r\n" +
				"Content-Length: " + postData.Length.ToString() + "\r\n\r\n" + postData;

			return getStringInternal(header, URL);
		}

		public string getFile(Hashtable post, string URL, string additionalHeaders)
		{
			string postData = makePostString(post);
			string header = "POST " + URL + " HTTP/1.1\r\n" +
				"Host: " + hostName + "\r\n" +
				"Accept: */*\r\n" +
				"Connection: Close\r\n" +
				getCookieString() + 
				"Content-Type: application/x-www-form-urlencoded\r\n" +
				additionalHeaders + 
				"Content-Length: " + postData.Length.ToString() + "\r\n\r\n" + postData;

			return getStringInternal(header, URL);
		}

		public string getFileWithReferer(Hashtable post, string URL, string referer)
		{
			string postData = makePostString(post);
			string header = "POST " + URL + " HTTP/1.1\r\n" +
				"Host: " + hostName + "\r\n" +
				"Accept: */*\r\n" +
				"Connection: Close\r\n" +
				getCookieString() + 
				"Content-Type: application/x-www-form-urlencoded\r\n" +
				"Referer: " +  referer + "\r\n" +
				"Content-Length: " + postData.Length.ToString() + "\r\n\r\n" + postData;

			return getStringInternal(header, URL);

		}

		private string makePostString(Hashtable post)
		{
			string postData = "";
			foreach(DictionaryEntry entry in post)
			{
				postData += entry.Key + "=" + entry.Value + "&";
			}
			postData = postData.Substring(0, postData.Length - 1);
			postData = postData.Replace("/", "%2F");
			return postData;

		}

		private string getCookieString()
		{
			string ret = "";
			
			if(cookies.Count != 0)
			{
				ret = "Cookie:";
				foreach(DictionaryEntry entry in cookies)
				{
					ret += " " + entry.Key + "=" + entry.Value + ";";
				}
				ret = ret.Substring(0, ret.Length - 1);
				ret += "\r\n";
			}

			return ret;
		}

		private string getStringInternal(string header, string URL)
		{
			//Sets up variables and a string to write to the server
			Encoding ASCII = Encoding.ASCII;
			//string Get = "GET " + URL + " HTTP/1.1\r\nHost: " + hostName + "\r\nConnection: Close\r\n" + header + "\r\n";
			Byte[] ByteHeader = ASCII.GetBytes(header);
			Byte[] RecvBytes = new Byte[256];
			String strRetPage = null;
 
			//Creates the Socket for sending data over TCP.
			Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream,
				ProtocolType.Tcp );

			// Connects to the host using IPEndPoint.
			s.Connect(hostEndpoint);
			if (!s.Connected)
			{
				strRetPage = "Unable to connect to host";
				return strRetPage;
			}

			// Sends the GET text to the host.
			s.Send(ByteHeader, ByteHeader.Length, SocketFlags.None);

			// Receives the page, looping until all bytes are received
			Int32 bytes = s.Receive(RecvBytes, RecvBytes.Length, 0);
			//strRetPage = "Default HTML page on " + server + ":\r\n";
			strRetPage = ASCII.GetString(RecvBytes, 0, bytes);
			
			while( (bytes > 0) && (strRetPage.IndexOf("\r\n\r\n") == -1) )
			{
				bytes = s.Receive(RecvBytes, RecvBytes.Length, SocketFlags.None);
				strRetPage = strRetPage + ASCII.GetString(RecvBytes, 0, bytes);
			}

			// Make sure we got the whole thing

			int pos = strRetPage.IndexOf("\r\n\r\n") + 4;
			string httpheader  = strRetPage.Substring(0, pos);
			parseCookies(httpheader);
			int startLen = httpheader.IndexOf("Content-Length: ", 0) + "Content-Length: ".Length;
			int endLen = strRetPage.IndexOf("\r\n", startLen);
			string lenStr = strRetPage.Substring(startLen, endLen - startLen);

			int contentLength = int.Parse(lenStr, System.Globalization.NumberStyles.Integer);

			int i = strRetPage.Length;
			
			//System.Console.Out.WriteLine(i);
			//System.Console.Out.WriteLine(contentLength + pos);

			//System.Console.Out.WriteLine(i < contentLength + pos);
			

			while(i < contentLength + pos)
			{
				bytes = s.Receive(RecvBytes, RecvBytes.Length, SocketFlags.None);
				strRetPage = strRetPage + ASCII.GetString(RecvBytes, 0, bytes);
				i = strRetPage.Length;
				//System.Console.Out.WriteLine(i);
				//System.Console.Out.WriteLine(contentLength + pos);
			}

			//System.Console.Out.WriteLine(strRetPage);


			return strRetPage;
		}

		private Byte[] getBinaryInternal(string header, string URL)
		{
			//Sets up variables and a string to write to the server
			Encoding ASCII = Encoding.ASCII;
			Byte[] ByteHeader = ASCII.GetBytes(header);
			Byte[] RecvBytes = new Byte[1024];
			Byte[] RetBytes;
			int curLength = 0;
			//String strRetPage = null;
 
			//Creates the Socket for sending data over TCP.
			Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream,
				ProtocolType.Tcp );

			// Connects to the host using IPEndPoint.
			s.Connect(hostEndpoint);
			
			//if (!s.Connected)
			//{
				//strRetPage = "Unable to connect to host";
			//	return new Byte[0];
			//}

			// Sends the GET text to the host.
			s.Send(ByteHeader, ByteHeader.Length, SocketFlags.None);

			// Receives the page, looping until all bytes are received
			Int32 bytes = s.Receive(RecvBytes, RecvBytes.Length, 0);
			curLength += bytes;
            
			RetBytes = new Byte[curLength];

			if(curLength < 1024)
			{
				for(int i = 0; i < curLength; i++)
				{
					RetBytes[i] = RecvBytes[i];
				}
			} 
			else 
			{
				RecvBytes.CopyTo(RetBytes, 0);
			}

			//strRetPage = ASCII.GetString(RecvBytes, 0, bytes);
			
			while (bytes > 0)
			{
				bytes = s.Receive(RecvBytes, RecvBytes.Length, SocketFlags.None);
				curLength = curLength + bytes;
				Byte[] temp = new Byte[curLength];
				RetBytes.CopyTo(temp, 0);
				if(bytes == RecvBytes.Length)
				{
					RecvBytes.CopyTo(temp, curLength - bytes);
				} 
				else 
				{
					for(int i = 0; i < bytes; i ++)
					{
						temp[curLength - bytes + i] = RecvBytes[i];
					}
				}
				
				
				RetBytes = temp;
				//strRetPage = strRetPage + ASCII.GetString(RecvBytes, 0, bytes);
			}



			string str1 = ASCII.GetString(RetBytes);
			int pos = str1.IndexOf("\r\n\r\n") + 4;
			string httpheader = str1.Substring(0, pos);
			parseCookies(httpheader);
			pos = str1.IndexOf("\r\n", pos) + 2;
			int len = RetBytes.Length - pos;

			Byte []FinalBytes = new Byte[len];
			for(int i = 0; i < len; i++)
			{
				FinalBytes[i] = RetBytes[i + pos];
			}
			int pos1 = 0;
			while(true)
			{
				pos1 = str1.IndexOf("Set-Cookie: ", pos1) + 12;
				if(pos1 == 11) break; // wasn't found, break out of loop
				int pos2 = str1.IndexOf("=", pos1 + 1);
				string key = str1.Substring(pos1, pos2-pos1);
				pos1 = str1.IndexOf(";", pos2 + 1);
				string val = str1.Substring(pos2 + 1, pos1 - (pos2 + 1));
				cookies[key] = val;
			}
/*
			while(1) 
			{
				int pos1 = str1.
				pos1 = str1.Find("Set-Cookie: ",pos1) + 12;
				if(pos1==11) break;
				pos2 = s.Find('=',pos1+1);
				key = s.Mid(pos1,pos2-pos1);
				pos1 = s.Find(';',pos2+1);
				val=s.Mid(pos2+1,pos1-(pos2+1));
			}
*/

			return FinalBytes;

		}


		void parseCookies(String header)
		{
			PageParser p = new PageParser(header);
					
			while(true)
			{
				string cookieStr = p.getContained("Set-Cookie: ", "\r\n");
				if(cookieStr == "") return;
				int pos = cookieStr.IndexOf("=");
				string key = cookieStr.Substring(0, pos).Trim();
				int pos2 = cookieStr.IndexOf(";", pos);
				string val = cookieStr.Substring(pos + 1, pos2 - pos - 1);
				cookies[key] = val;
			}

		}
	}
}
